/* Base */

:root {
  /* Colors */
  --primary-bg-color: #fafafa;

  --alert-standard-color: #666666;
  --alert-standard-color-bg: #e6e6e6;
  --alert-info-color: #00539f;
  --alert-info-color-bg: #E6EEF5;
  --alert-success-color: #4CAF50;
  --alert-success-color-bg: #EDF7EE;
  --alert-warning-color: #FF9800;
  --alert-warning-color-bg: #FFF5E6;
  --alert-error-color: #D93025;
  --alert-error-color-bg: #FAEFEB;
}

body {
  display: flex;
  min-height: 100vh;
  flex-direction: column;
  background-color: var(--primary-bg-color);
}

/* Layout */
.valign-wrapper {
  position: absolute;
  width:100vw;
  height:100vh;
}

.valign {
  width: 100%;
}

main {
  flex: 1 0 auto;
}

/* Alerts & Errors */
.alert {
  width: 100%;
  margin: 10px auto;
  padding: 10px;
  border-radius: 2px;
  border-left-width: 4px;
  border-left-style: solid;
}

.alert .title {
  font-weight: bold;
}

.alert .title::after {
  content: "\A";
  white-space: pre;
}

.alert.alert-error {
  color: var(--alert-error-color);
  border-left-color: var(--alert-error-color);
  background-color: var(--alert-error-color-bg);
}

.card.card-error, .card.status-offline {
  border-top: 4px solid var(--alert-error-color);
}

.card.status-online {
  border-top: 4px solid var(--alert-success-color);
}

.card.status-not-responding {
  border-top: 4px solid var(--alert-warning-color);
}

.card.status-unknown {
  border-top: 4px solid var(--alert-standard-color);
}

/* Login Page */
#login-page .row.no-bottom-margin {
  margin-bottom: 0 !important;
}

#login-page .logo {
  display: block;
  width: auto;
  max-width: 300px;
  margin-left: auto;
  margin-right: auto;
}

#login-page .input-field input:focus + label  {
  color: #000;
}

#login-page .input-field input:focus {
  border-bottom: 1px solid #000;
  box-shadow: 0 1px 0 0 #000;
}

#login-page .input-field .prefix.active {
  color: #000;
}

#login-page .version-number {
  display: block;
  text-align: center;
  margin-bottom: 20px;;
  color:#808080;
  font-size: 12px;
}

#login-page footer {
  color: #757575;
  font-size: 12px;
}

#login-page footer a {
  color: #424242;
}

#login-page footer p {
  -webkit-margin-before: 0px;
  margin-block-start: 0px;
  -webkit-margin-after: 5px;
  margin-block-end: 5px;
}

#login-page footer p:last-child {
  -webkit-margin-after: 0px;
  margin-block-end: 0px;
}

/* Dashboard */
.logo-wrapper {
  height: 64px;
  height: 100%;
  width: 0;
  margin-left: 24px;
}

.logo {
  width: auto;
  height: 48px;
  margin: 8px 0;
}

@media only screen and (max-width: 601px) {
  .logo {
    height: 38px;
    margin: 9px 0;
  }
}

.nav-icons {
  margin-right: 24px;
}

.nav-icons i {
  color: black;
}

nav {
  height: auto;
  line-height: normal;
}

.select-port-container {
  margin-top: 8px;
  margin-right: 10px;
  width: 350px;
}

.serial-port-select {
  margin-top: 8px;
  margin-right: 10px;
  width: 350px;
}

.serial-port-select .select-dropdown {
  color: black;
}

.serial-port-select .select-dropdown:focus {
  border-bottom: 1px solid #607d8b !important;
}

.serial-port-select .caret {
  fill: black;
}

.serial-port-select .dropdown-content li>span {
  color: black;
}

#nav-dropdown li a,  .node-dropdown li a {
  color: black;
}

main .container {
  margin-top: 20px;
  margin-bottom: 20px;
  width: 90%;
  max-width: 1920px;
}

#nodes .card-content {
  height: calc(100% - 47px);
}

#nodes .card-content, #nodes .card-action  {
  padding: 12px;
}

#nodes .grid-1-col {
  display: grid;
  grid-template-columns: 1fr;
}

#nodes .grid-2-col {
  display: grid;
  grid-template-columns: 1fr 1fr;
  grid-column-gap: 1.5rem;
}

#nodes .grid-3-col {
  display: grid;
  grid-template-columns: 1fr 1fr 1fr;
  grid-column-gap: 1.5rem;
}

@media only screen and (max-width: 1100px) {
  #nodes .grid-3-col {
    grid-template-columns: 1fr 1fr;
    grid-column-gap: 1.5rem;
  }
}

@media only screen and (max-width: 750px) {
  #nodes .grid-2-col {
    grid-template-columns: 1fr;
    grid-column-gap: 0;
  }

  #nodes .grid-3-col {
    grid-template-columns: 1fr;
    grid-column-gap: 0;
  }
}

i.node-update-avaliable {
  color:#3f51b5;
}

i.node-webserver {
  color:#039be5;
}

.node-config-path {
  margin-top: -8px;
  margin-bottom: 8px;
  font-size: 14px;
}

.node-card-comment {
  color: #444;
  font-style: italic;
}

.card-action a, .card-dropdown-action a {
  cursor: pointer;
}

.tooltipped {
  cursor: help;
}

#js-loading-indicator {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
}

.editor {
  margin-top: 0;
  margin-bottom: 0;
  border-radius: 3px;
  height: calc(100% - 56px);
}

.inlinecode {
  box-sizing: border-box;
  padding: 0.2em 0.4em;
  margin: 0;
  font-size: 85%;
  background-color: rgba(27,31,35,0.05);
  border-radius: 3px;
  font-family: "SFMono-Regular",Consolas,"Liberation Mono",Menlo,Courier,monospace;
}

.log {
  height: 100%;
  max-height: calc(100% - 56px);
  background-color: #1c1c1c;
  margin-top: 0;
  margin-bottom: 0;
  font-family: "SFMono-Regular", Consolas, "Liberation Mono", Menlo, Courier, monospace;
  font-size: 12px;
  padding: 16px;
  overflow: auto;
  line-height: 1.45;
  border-radius: 3px;
  white-space: pre-wrap;
  overflow-wrap: break-word;
  color: #DDD;
}

.log-bold { font-weight: bold; }
.log-italic { font-style: italic; }
.log-underline { text-decoration: underline; }
.log-strikethrough { text-decoration: line-through; }
.log-underline.log-strikethrough { text-decoration: underline line-through; }
.log-secret {
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
.log-secret-redacted {
  opacity: 0;
  width: 1px;
  font-size: 1px;
}
.log-fg-black { color: rgb(128,128,128); }
.log-fg-red { color: rgb(255,0,0); }
.log-fg-green { color: rgb(0,255,0); }
.log-fg-yellow { color: rgb(255,255,0); }
.log-fg-blue { color: rgb(0,0,255); }
.log-fg-magenta { color: rgb(255,0,255); }
.log-fg-cyan { color: rgb(0,255,255); }
.log-fg-white { color: rgb(187,187,187); }
.log-bg-black { background-color: rgb(0,0,0); }
.log-bg-red { background-color: rgb(255,0,0); }
.log-bg-green { background-color: rgb(0,255,0); }
.log-bg-yellow { background-color: rgb(255,255,0); }
.log-bg-blue { background-color: rgb(0,0,255); }
.log-bg-magenta { background-color: rgb(255,0,255); }
.log-bg-cyan { background-color: rgb(0,255,255); }
.log-bg-white { background-color: rgb(255,255,255); }

ul.browser-default {
  padding-left: 30px;
  margin-top: 10px;
  margin-bottom: 15px;
}

ul.browser-default li {
  list-style-type: initial;
}

ul.stepper:not(.horizontal) .step.active::before, ul.stepper:not(.horizontal) .step.done::before, ul.stepper.horizontal .step.active .step-title::before, ul.stepper.horizontal .step.done .step-title::before {
  background-color: #3f51b5 !important;
}

.select-action {
  width: auto !important;
  height: auto !important;
  white-space: nowrap;
}

.modal {
  width: 95%;
  max-height: 90%;
  height: 85% !important;
}

.page-footer {
  display: flex;
  align-items: center;
  min-height: 50px;
  padding-top: 0;
  color: grey;
}

.page-footer a {
  color: #afafaf;
}

@media only screen and (max-width: 992px) {
  .page-footer .left, .page-footer .right {
    width: 100%;
    text-align: center;
  }
}
